#include <string.h>
#include <windows.h>
#include <winsock.h>
#include "mailbox.h"
#include "acidmail.h"
#include "version.h"
#include "resource.h"
#include "..\litestep\lsapi\lsapi.h"

BOOL CALLBACK GetPassProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM /* lParam */);
extern HWND hwndMain;
extern HINSTANCE hwndInstance;
char temp_name[64] = {0}, temp_pass[64] = {0};

Mailbox::Mailbox(char* name, char* type, char* server, char* user, char* pass)
{
	// Copy args
	strcpy(sname, name);
	strcpy(stype, type);
	strcpy(suser, user);
	char* p = strtok(server, ":");
	strcpy(sserver, p);
	p = strtok(NULL, "");
	if (p) port = atoi(p);
	else if (!strcmp("imap", stype)) port = 143;
	else port = 110;
	strcpy(spass, pass);

	if (*pass)
		strcpy(spass, pass);
	else
	{
		char temp[256];
		sprintf(temp, "%s/%s", sserver, suser);
		GetProfileString("AcidMail", temp, "", temp, 256);
        if (!strlen(temp))
		{
			strcpy(temp_name, sname);
            DialogBox(hwndInstance, MAKEINTRESOURCE(IDD_GETPASS), hwndMain, GetPassProc);			
            if (strlen(temp_pass)) 
			{
				strcpy(spass, temp_pass);
				sprintf(temp, "%s/%s", sserver, suser);
				WriteProfileString("AcidMail", temp, Encrypt(spass));
			}
			memset(&(temp_pass), 0, sizeof(temp_pass));
		}
		else
			strcpy(spass, Encrypt(temp));
	}

	// Init other vars
	mail.iNumMail = 0;
	mail.bError = false;

	// Set evars
	strcpy(sevar, "AcidMail");
	strcat(sevar, name);
	LSSetVariable(sevar, "0");
}

Mailbox::~Mailbox()
{

}

Mail Mailbox::CheckMail()
{
	SOCKET s;
	try
	{
		WSADATA wsa;
		if (WSAStartup(MAKEWORD(1,1),&wsa))
			throw 1;	
		HOSTENT*  dns;
		if (isalpha(sserver[0]))
			dns = gethostbyname(sserver);
		else
		{
			unsigned int addr = inet_addr(sserver); 
			dns = gethostbyaddr((char *)&addr,4,AF_INET);
		}
		if(!dns) throw 2; 
		char* &ip=dns->h_addr;
		sockaddr_in  a={AF_INET,htons(port), ip[0],ip[1],ip[2],ip[3]};
		s=socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
		if(connect(s,(sockaddr*)&a,sizeof(a)) <0 ) throw 6;
		if (!strcmp("pop3", stype))
			mail.iNumMail = Pop3Check(s);
		else if(!strcmp("imap", stype))
			mail.iNumMail = ImapCheck(s);
		else
			throw 9;
		closesocket(s);
		WSACleanup();
		char snum[8];
		sprintf(snum, "%d", mail.iNumMail);
		LSSetVariable(sevar, snum);
		mail.bError = false;
	}
	catch (int e)
	{
		ErrorHandler(e, s);
		mail.bError = true;
	}
	return mail;
}

int Mailbox::Pop3Check(const SOCKET& s)
{
	char data[128];
	char* buf = data;
	int size=128;

	sprintf(buf, "");
	if(send(s, buf, static_cast<int>(strlen(buf)), 0) < 0) throw 7;
	if(recv(s, buf, size, 0) <= 0 || *buf != '+' ) throw 7;

	sprintf(buf, "user %s\r\n" ,suser);
	if(send(s, buf, static_cast<int>(strlen(buf)), 0) <= 0) throw 7;
	if(recv(s, buf, size, 0) <= 0) throw 7;
	if(*buf != '+') throw 8;

	sprintf(buf, "pass %s\r\n" ,spass);
	if(send(s, buf, static_cast<int>(strlen(buf)), 0) <= 0) throw 7;
	if(recv(s, buf, size, 0) <= 0) throw 7;
	if(*buf != '+') throw 8;

	sprintf(buf, "stat\r\n");
	if(send(s, buf, static_cast<int>(strlen(buf)), 0) <= 0) throw 7;
	if(recv(s, buf, size, 0) <= 0 || *buf != '+' ) throw 7;

	send(s, "quit\r\n", 6, 0);
	char test[6];
	strcpy(test, strtok(buf, " "));
	unsigned int ans;
	if(test != NULL)
		ans = atoi(strtok(NULL, " "));
	else
		throw 255;
	recv(s, buf, size, 0);
	return ans;
}

int Mailbox::ImapCheck(const SOCKET& s)
{
	char data[1024];
	char* buf = data;
	int size=1024;
	unsigned int ans;

	sprintf(buf, "");
	if(send(s, buf, static_cast<int>(strlen(buf)), 0) < 0) throw 7;
	if(recv(s, buf, size, 0) <= 0) throw 7;

	sprintf(buf, "a01 login %s %s\r\n", suser, spass);
	if(send(s, buf, static_cast<int>(strlen(buf)), 0) <= 0) throw 7;
	if(recv(s, buf, size, 0) <= 0) throw 7;

	sprintf(buf, "a02 examine inbox\r\n");
	if(send(s, buf, static_cast<int>(strlen(buf)), 0) <= 0) throw 7;
	if(recv(s, buf, size, 0) <= 0) throw 7;

	char testbuf[256];
	if(strstr(buf, "RECENT"))
	{
		strcpy(testbuf, strtok(buf, "*"));
		strcpy(testbuf, strtok(NULL, "*"));
		strcpy(testbuf, strtok(NULL, "*"));
		ans = atoi(strtok(NULL, " "));
	}
	else
		throw 255;
	send(s, "a03 logout\r\n", 12, 0);
	recv(s, buf, size, 0);
	return ans;
}

void Mailbox::ClearMail()
{
	mail.iNumMail = 0;
	mail.bError = false;
	LSSetVariable(sevar, "0");
}

void Mailbox::ErrorHandler(const int& errnum, const SOCKET& s)
{
	if (errnum > 5)
	{
		closesocket(s);
		WSACleanup();
	}	
	switch(errnum)
	{
		case 1:
			LSLogPrintf(LOG_ERROR, szAppName, "Could not initialize winsock.");
			break;
		case 2:
			LSLogPrintf(LOG_WARNING, szAppName, "Could not find host: %s", sserver);
			break;
		case 6:
			LSLogPrintf(LOG_WARNING, szAppName, "Could not connect to host: %s", sserver);
			break;
		case 7:
			LSLogPrintf(LOG_WARNING, szAppName, "Connection error with mailbox: %s", sname);
			break;
		case 8:
			LSLogPrintf(LOG_WARNING, szAppName, "Invalid username/password");
			break;
		case 9:
			LSLogPrintf(LOG_WARNING, szAppName, "Invalid mailbox type");
			break;
		default:
			LSLogPrintf(LOG_WARNING, szAppName, "An error accured with mailbox: %s", sname);
	}
}

char* Mailbox::Encrypt(char* pass)
{
	char* ret=_strdup(pass);
	char* p=ret;
	for (; *pass; *pass++, *ret++)
	{
		*ret=*pass ^ 0x5405;
	}
	return p;
}

BOOL CALLBACK GetPassProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM /* lParam */)
{
	switch (msg)
	{
		case WM_INITDIALOG: 
		{
			char temp[256] = "";
			sprintf(temp, "%s Password", temp_name);
			SetWindowText(hwnd, temp);
			return TRUE;
		}

		case WM_COMMAND:
		{
			if (HIWORD(wParam) == BN_CLICKED)
			{
				switch (LOWORD(wParam))
				{
					case IDOK:
					{
						GetDlgItemText(hwnd, IDC_PASSWORD, temp_pass, 64);
					}
					case IDCANCEL:
					{
						EndDialog(hwnd, 0);
					}
					break;
				}
			}
		}
		break;
	}

	return false;
}
