#include <windows.h>
#include "..\litestep\lsapi\lsapi.h"
#include "main.h"
#include "version.h"

BOOL loaded = FALSE;

// Litestep calls this function after it loads the module. This is where the
// module should register bang commands and create windows. If initialization
// succeeds, return zero. If an error occurs, return a nonzero value.
extern "C" int initModuleEx(HWND hParent, HINSTANCE hInstance, const char *path)
{   
	// Ignore attempts to load the module more than once
	if (loaded)
	{
		return 1;
	}
	
	hwndParent = hParent;
	hwndInstance = hInstance;
	
	//Create window for message handling
	WNDCLASS wc = { 0 };
	wc.lpfnWndProc = WndProc;
	wc.hInstance = hInstance;
	wc.lpszClassName = szAppName;

	RegisterClass(&wc);
	hwndMain = CreateWindowEx(WS_EX_TOOLWINDOW, szAppName, NULL, WS_POPUP, 0, 0, 0, 0, NULL, NULL, hInstance, NULL);

	//Register messages to Litestep
	SendMessage(hwndParent, LM_REGISTERMESSAGE, (WPARAM)hwndMain, (LPARAM)msgs);

	acidmail = new Acidmail();
	loaded = TRUE;

	return 0;
}

// Litestep calls this function before it unloads the module. This function
// should unregister bang commands and destroy windows.
extern "C" void quitModule(HINSTANCE hInstance)
{   
	//Unregister messages to Litestep
	SendMessage(hwndParent, LM_UNREGISTERMESSAGE, (WPARAM)hwndMain, (LPARAM)msgs);

	DestroyWindow(hwndMain);
	UnregisterClass(szAppName, hInstance);

	delete acidmail;
	acidmail = NULL;
}

// Message processor
LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	LPSTR buf = (LPSTR)lParam;
	switch (msg)
	{
		case LM_GETREVID:	
			if (wParam == 0 || wParam == 1)
			{				
				strcpy(buf, szVersion);
			}
			else
			{
				strcpy(buf, "");
			}
            return strlen(buf);
			break;
		
		case LM_REFRESH:
			delete acidmail;
			acidmail = new Acidmail();
			break;
		
		case WM_TIMER:
            acidmail->CheckMail();
			break;
	}
	return DefWindowProc(hwnd, msg, wParam, lParam);
}

BOOL __stdcall _DllMainCRTStartup(HINSTANCE hInst, DWORD fdwReason, LPVOID lpvRes)
{
	// We don't need thread notifications for what we're doing.  Thus, get
	// rid of them, thereby eliminating some of the overhead of this DLL
	DisableThreadLibraryCalls(hInst);

	return TRUE;
}
